@@ -13,19 +13,20 @@
 # limitations under the License.
 
 import time
+import requests
+import logging
 from kubernetes import client, config
+from urllib.parse import urlparse
 
 from ..constants import constants
 from ..utils import utils
 from .creds_utils import set_gcs_credentials, set_s3_credentials, set_azure_credentials
-from .kf_serving_watch import watch as isvc_watch
-from ..models.v1alpha2_inference_service import V1alpha2InferenceService
-from ..models.v1alpha2_inference_service_spec import V1alpha2InferenceServiceSpec
+from .kf_serving_watch import isvc_watch
 
 
 class KFServingClient(object):
 
-    def __init__(self, config_file=None, context=None, # pylint: disable=too-many-arguments
+    def __init__(self, config_file=None, context=None,  # pylint: disable=too-many-arguments
                  client_configuration=None, persist_config=True):
         """
         KFServing client constructor
@@ -85,7 +86,7 @@ class KFServingClient(object):
             raise RuntimeError("Invalid storage_type: %s, only support GCS, S3 and Azure\
                 currently.\n" % storage_type)
 
-    def create(self, inferenceservice, namespace=None, watch=False, timeout_seconds=600): #pylint:disable=inconsistent-return-statements
+    def create(self, inferenceservice, namespace=None, watch=False, timeout_seconds=600):  # pylint:disable=inconsistent-return-statements
         """
         Create the inference service
         :param inferenceservice: inference service object
@@ -95,13 +96,17 @@ class KFServingClient(object):
         :return: created inference service
         """
 
+        if inferenceservice.api_version == constants.KFSERVING_V1ALPHA2:
+            logging.warning("The version v1alpha2 will be deprecated from KFServing 0.6 release.")
+        version = inferenceservice.api_version.split("/")[1]
+
         if namespace is None:
             namespace = utils.set_isvc_namespace(inferenceservice)
 
         try:
             outputs = self.api_instance.create_namespaced_custom_object(
                 constants.KFSERVING_GROUP,
-                constants.KFSERVING_VERSION,
+                version,
                 namespace,
                 constants.KFSERVING_PLURAL,
                 inferenceservice)
@@ -118,15 +123,19 @@ class KFServingClient(object):
         else:
             return outputs
 
-    def get(self, name=None, namespace=None, watch=False, timeout_seconds=600): #pylint:disable=inconsistent-return-statements
+    def get(self, name=None, namespace=None, watch=False, timeout_seconds=600, version=constants.KFSERVING_V1BETA1_VERSION):  # pylint:disable=inconsistent-return-statements
         """
         Get the inference service
         :param name: existing inference service name
         :param namespace: defaults to current or default namespace
         :param watch: True to watch the service until timeout elapsed or status is ready
         :param timeout_seconds: timeout seconds for watch, default to 600s
+        :param version: api group version
         :return: inference service
         """
+        if version == 'v1alpha2':
+            logging.warning("The version v1alpha2 will be deprecated from KFServing 0.6 release.")
+
         if namespace is None:
             namespace = utils.get_default_target_namespace()
 
@@ -140,7 +149,7 @@ class KFServingClient(object):
                 try:
                     return self.api_instance.get_namespaced_custom_object(
                         constants.KFSERVING_GROUP,
-                        constants.KFSERVING_VERSION,
+                        version,
                         namespace,
                         constants.KFSERVING_PLURAL,
                         name)
@@ -157,7 +166,7 @@ class KFServingClient(object):
                 try:
                     return self.api_instance.list_namespaced_custom_object(
                         constants.KFSERVING_GROUP,
-                        constants.KFSERVING_VERSION,
+                        version,
                         namespace,
                         constants.KFSERVING_PLURAL)
                 except client.rest.ApiException as e:
@@ -165,7 +174,7 @@ class KFServingClient(object):
                         "Exception when calling CustomObjectsApi->list_namespaced_custom_object:\
                         %s\n" % e)
 
-    def patch(self, name, inferenceservice, namespace=None, watch=False, timeout_seconds=600): # pylint:disable=too-many-arguments,inconsistent-return-statements
+    def patch(self, name, inferenceservice, namespace=None, watch=False, timeout_seconds=600):  # pylint:disable=too-many-arguments,inconsistent-return-statements
         """
         Patch existing inference service
         :param name: existing inference service name
@@ -175,13 +184,17 @@ class KFServingClient(object):
         :param timeout_seconds: timeout seconds for watch, default to 600s
         :return: patched inference service
         """
+
+        if inferenceservice.api_version == constants.KFSERVING_V1ALPHA2:
+            logging.warning("The version v1alpha2 will be deprecated from KFServing 0.6 release.")
+        version = inferenceservice.api_version.split("/")[1]
         if namespace is None:
             namespace = utils.set_isvc_namespace(inferenceservice)
 
         try:
             outputs = self.api_instance.patch_namespaced_custom_object(
                 constants.KFSERVING_GROUP,
-                constants.KFSERVING_VERSION,
+                version,
                 namespace,
                 constants.KFSERVING_PLURAL,
                 name,
@@ -201,7 +214,7 @@ class KFServingClient(object):
         else:
             return outputs
 
-    def replace(self, name, inferenceservice, namespace=None, watch=False, timeout_seconds=600): # pylint:disable=too-many-arguments,inconsistent-return-statements
+    def replace(self, name, inferenceservice, namespace=None, watch=False, timeout_seconds=600):  # pylint:disable=too-many-arguments,inconsistent-return-statements
         """
         Replace the existing inference service
         :param name: existing inference service name
@@ -212,6 +225,10 @@ class KFServingClient(object):
         :return: replaced inference service
         """
 
+        if inferenceservice.api_version == constants.KFSERVING_V1ALPHA2:
+            logging.warning("The version v1alpha2 will be deprecated from KFServing 0.6 release.")
+        version = inferenceservice.api_version.split("/")[1]
+
         if namespace is None:
             namespace = utils.set_isvc_namespace(inferenceservice)
 
@@ -223,7 +240,7 @@ class KFServingClient(object):
         try:
             outputs = self.api_instance.replace_namespaced_custom_object(
                 constants.KFSERVING_GROUP,
-                constants.KFSERVING_VERSION,
+                version,
                 namespace,
                 constants.KFSERVING_PLURAL,
                 name,
@@ -241,80 +258,12 @@ class KFServingClient(object):
         else:
             return outputs
 
-    def rollout_canary(self, name, percent, namespace=None, # pylint:disable=too-many-arguments,inconsistent-return-statements
-                       canary=None, watch=False, timeout_seconds=600):
-        """
-        Rollout the canary endpoint
-        :param name: inference service name
-        :param percent: traffic percentage to the canary endpoint
-        :param namespace: defaults to current or default namespace
-        :param canary: canary endpoint spec
-        :param watch: True to watch the service until timeout elapsed or status is ready
-        :param timeout_seconds: timeout seconds for watch, default to 600s
-        :return: inference service with canary endpoint
-        """
-
-        if namespace is None:
-            namespace = utils.get_default_target_namespace()
-
-        current_isvc = self.get(name, namespace=namespace)
-
-        if canary is None and 'canary' not in current_isvc['spec']:
-            raise RuntimeError("Canary spec missing? Specify canary for the InferenceService.")
-
-        current_isvc['spec']['canaryTrafficPercent'] = percent
-        if canary:
-            current_isvc['spec']['canary'] = canary
-
-        return self.patch(name=name, inferenceservice=current_isvc, namespace=namespace,
-                          watch=watch, timeout_seconds=timeout_seconds)
-
-    def promote(self, name, namespace=None, watch=False, timeout_seconds=600): # pylint:disable=too-many-arguments,inconsistent-return-statements
-        """
-        Promote canary endpoint to default
-        :param name: inference service name
-        :param namespace: defaults to current or default namespace
-        :param watch: True to watch the service until timeout elapsed or status is ready
-        :param timeout_seconds: timeout seconds for watch, default to 600s
-        :return: promoted inference service
-        """
-
-        if namespace is None:
-            namespace = utils.get_default_target_namespace()
-
-        current_isvc = self.get(name, namespace=namespace)
-        api_version = current_isvc['apiVersion']
-
-        try:
-            current_canary_spec = current_isvc['spec']['canary']
-        except KeyError:
-            raise RuntimeError("Cannot promote a InferenceService that has no Canary Spec.")
-
-        try:
-            annotations = current_isvc['metadata']['annotations']
-        except KeyError:
-            annotations = dict()
-
-        inferenceservice = V1alpha2InferenceService(
-            api_version=api_version,
-            kind=constants.KFSERVING_KIND,
-            metadata=client.V1ObjectMeta(
-                name=name,
-                namespace=namespace,
-                annotations=annotations),
-            spec=V1alpha2InferenceServiceSpec(
-                default=current_canary_spec,
-                canary=None,
-                canary_traffic_percent=0))
-
-        return self.replace(name=name, inferenceservice=inferenceservice, namespace=namespace,
-                            watch=watch, timeout_seconds=timeout_seconds)
-
-    def delete(self, name, namespace=None):
+    def delete(self, name, namespace=None, version=constants.KFSERVING_VERSION):
         """
         Delete the inference service
         :param name: inference service name
         :param namespace: defaults to current or default namespace
+        :param version: api group version
         :return:
         """
         if namespace is None:
@@ -323,25 +272,24 @@ class KFServingClient(object):
         try:
             return self.api_instance.delete_namespaced_custom_object(
                 constants.KFSERVING_GROUP,
-                constants.KFSERVING_VERSION,
+                version,
                 namespace,
                 constants.KFSERVING_PLURAL,
-                name,
-                client.V1DeleteOptions())
+                name)
         except client.rest.ApiException as e:
             raise RuntimeError(
                 "Exception when calling CustomObjectsApi->delete_namespaced_custom_object:\
                  %s\n" % e)
 
-
-    def is_isvc_ready(self, name, namespace=None): #pylint:disable=inconsistent-return-statements
+    def is_isvc_ready(self, name, namespace=None, version=constants.KFSERVING_V1BETA1_VERSION):  # pylint:disable=inconsistent-return-statements
         """
         Check if the inference service is ready.
         :param name: inference service name
         :param namespace: defaults to current or default namespace
         :return:
         """
-        kfsvc_status = self.get(name, namespace=namespace)
+        kfsvc_status = self.get(name, namespace=namespace,
+                                version=version)
         status = 'Unknown'
         for condition in kfsvc_status['status'].get('conditions', {}):
             if condition.get('type', '') == 'Ready':
@@ -349,11 +297,11 @@ class KFServingClient(object):
                 return status.lower() == "true"
         return False
 
-
-    def wait_isvc_ready(self, name, namespace=None, #pylint:disable=too-many-arguments
+    def wait_isvc_ready(self, name, namespace=None,  # pylint:disable=too-many-arguments
                         watch=False,
                         timeout_seconds=600,
-                        polling_interval=10):
+                        polling_interval=10,
+                        version=constants.KFSERVING_V1BETA1_VERSION):
         """
         Waiting for inference service ready, print out the inference service if timeout.
         :param name: inference service name
@@ -362,6 +310,7 @@ class KFServingClient(object):
         :param timeout_seconds: timeout seconds for waiting, default to 600s.
                Print out the InferenceService if timeout.
         :param polling_interval: The time interval to poll status
+        :param version: api group version
         :return:
         """
         if watch:
@@ -372,9 +321,97 @@ class KFServingClient(object):
         else:
             for _ in range(round(timeout_seconds/polling_interval)):
                 time.sleep(polling_interval)
-                if self.is_isvc_ready(name, namespace=namespace):
+                if self.is_isvc_ready(name, namespace=namespace, version=version):
                     return
 
-            current_isvc = self.get(name, namespace=namespace)
+            current_isvc = self.get(name, namespace=namespace, version=version)
             raise RuntimeError("Timeout to start the InferenceService {}. \
                                The InferenceService is as following: {}".format(name, current_isvc))
+
+    def create_trained_model(self, trainedmodel, namespace):
+            """
+            Create a trained model
+            :param trainedmodel: trainedmodel object
+            :param namespace: defaults to current or default namespace
+            :return:
+            """
+            version = trainedmodel.api_version.split("/")[1]
+
+            try:
+                self.api_instance.create_namespaced_custom_object(
+                    constants.KFSERVING_GROUP,
+                    version,
+                    namespace,
+                    constants.KFSERVING_PLURAL_TRAINEDMODEL,
+                    trainedmodel)
+            except client.rest.ApiException as e:
+                raise RuntimeError(
+                    "Exception when calling CustomObjectsApi->create_namespaced_custom_object:\
+                     %s\n" % e)
+
+    def delete_trained_model(
+        self, name, namespace=None, version=constants.KFSERVING_V1ALPHA1_VERSION
+    ):
+        """
+        Delete the trained model
+        :param name: trained model name
+        :param namespace: defaults to current or default namespace
+        :param version: api group version
+        :return:
+        """
+        if namespace is None:
+            namespace = utils.get_default_target_namespace()
+
+        try:
+            return self.api_instance.delete_namespaced_custom_object(
+                constants.KFSERVING_GROUP,
+                version,
+                namespace,
+                constants.KFSERVING_PLURAL_TRAINEDMODEL,
+                name,
+            )
+        except client.rest.ApiException as e:
+            raise RuntimeError(
+                "Exception when calling CustomObjectsApi->delete_namespaced_custom_object:\
+                 %s\n"
+                % e
+            )
+
+    def wait_model_ready(self, service_name, model_name, isvc_namespace=None, # pylint:disable=too-many-arguments
+                         isvc_version=constants.KFSERVING_V1BETA1_VERSION,
+                         cluster_ip=None,
+                         protocol_version="v1",
+                         timeout_seconds=600,
+                         polling_interval=10):
+        """
+        Waiting for model to be ready to service, print out trained model if timeout.
+        :param service_name: inference service name
+        :param model_name: trained model name
+        :param isvc_namespace: defaults to current or default namespace of inference service
+        :param isvc_version: api group version of inference service
+        :param protocol_version: version of the dataplane protocol
+        :param cluster_ip: ip of the kuberenetes cluster
+        :param timeout_seconds: timeout seconds for waiting, default to 600s.
+          Print out the InferenceService if timeout.
+        :param polling_interval: The time interval to poll status
+        :return:
+        """
+        isvc = self.get(
+            service_name,
+            namespace=isvc_namespace,
+            version=isvc_version,
+        )
+
+        host = urlparse(isvc["status"]["url"]).netloc
+        headers = {"Host": host}
+
+        for _ in range(round(timeout_seconds/polling_interval)):
+            time.sleep(polling_interval)
+            # Check model health API
+            url = f"http://{cluster_ip}/{protocol_version}/models/{model_name}"
+            response = requests.get(url, headers=headers).status_code
+            if response == 200:
+                return
+
+        raise RuntimeError(f"InferenceService ({service_name}) has not loaded the \
+                            model ({model_name}) before the timeout.")
