@@ -18,8 +18,16 @@ import os
 KFSERVING_GROUP = 'serving.kubeflow.org'
 KFSERVING_KIND = 'InferenceService'
 KFSERVING_PLURAL = 'inferenceservices'
+KFSERVING_KIND_TRAINEDMODEL = 'TrainedModel'
+KFSERVING_PLURAL_TRAINEDMODEL = 'trainedmodels'
 KFSERVING_VERSION = os.environ.get('KFSERVING_VERSION', 'v1alpha2')
+KFSERVING_V1BETA1_VERSION = 'v1beta1'
+KFSERVING_V1ALPHA2_VERSION = 'v1alpha2'
+KFSERVING_V1ALPHA1_VERSION = "v1alpha1"
 KFSERVING_API_VERSION = KFSERVING_GROUP + '/' + KFSERVING_VERSION
+KFSERVING_V1BETA1 = KFSERVING_GROUP + '/' + KFSERVING_V1BETA1_VERSION
+KFSERVING_V1ALPHA2 = KFSERVING_GROUP + '/' + KFSERVING_V1ALPHA2_VERSION
+KFSERVING_V1ALPHA1 = KFSERVING_GROUP + '/' + KFSERVING_V1ALPHA1_VERSION
 
 KFSERVING_LOGLEVEL = os.environ.get('KFSERVING_LOGLEVEL', 'INFO').upper()
 
