@@ -18,6 +18,8 @@ import sys
 import json
 import tornado.web
 from tornado.httpclient import AsyncHTTPClient
+from cloudevents.http import CloudEvent
+from http import HTTPStatus
 
 PREDICTOR_URL_FORMAT = "http://{0}/v1/models/{1}:predict"
 EXPLAINER_URL_FORMAT = "http://{0}/v1/models/{1}:explain"
@@ -52,15 +54,33 @@ class KFModel:
 
     def preprocess(self, request: Dict) -> Dict:
         # If cloudevent dict, then parse 'data' field. Otherwise, pass through.
-        if  "data" in request \
-            and "time" in request \
-            and "type" in request \
-            and "source" in request \
-            and "id" in request \
-            and "specversion" in request:
-            return request["data"]
-        else:
-            return request
+        response = request
+
+        if(isinstance(request, CloudEvent)):
+            response = request.data
+            if(isinstance(response, bytes)):
+                try:
+                    response = json.loads(response.decode('UTF-8'))
+                except (json.decoder.JSONDecodeError, UnicodeDecodeError) as e:
+                    attributes = request._attributes
+                    if "content-type" in attributes:
+                        if attributes["content-type"] == "application/cloudevents+json" or attributes["content-type"] == "application/json":
+                            raise tornado.web.HTTPError(
+                                status_code=HTTPStatus.BAD_REQUEST,
+                                reason="Unrecognized request format: %s" % e
+                            )
+
+        elif(isinstance(request, dict)): #CE structured - https://github.com/cloudevents/sdk-python/blob/8773319279339b48ebfb7b856b722a2180458f5f/cloudevents/http/http_methods.py#L126 
+ 
+            if "time" in request \
+                and "type" in request \
+                and "source" in request \
+                and "id" in request \
+                and "specversion" in request \
+                and "data" in request:
+                response = request["data"]
+                
+        return response
 
     def postprocess(self, request: Dict) -> Dict:
         return request
